import {each} from "lodash";

import {fuse} from "../lib/utils";
import {getProps} from "../index";
import {button,font,color,options} from "../variables";
import {buttonSize,buttonVariant,buttonOutlineVariant} from "./mixins/buttons";
import transition from "./mixins/transition";
import boxShadow from "./mixins/box-shadow";

//
// Base styles
//

function buttonRule(themeProps = {}) {

  const {buttonProps,fontProps} = getProps(themeProps,{button,font});

  return fuse(
    {
      display: "inline-block",
      fontWeight: buttonProps.fontWeight,
      textAlign: "center",
      whiteSpace: "nowrap",
      verticalAlign: "middle",
      userSelect: "none",
      border: `${buttonProps.inputBorderWidth} solid transparent`,
    },
    buttonSize(themeProps,buttonProps.inputPaddingY, buttonProps.inputPaddingX, fontProps.sizeBase, buttonProps.inputLineHeight, buttonProps.borderRadius),
    transition(themeProps,buttonProps.transition),


    {
      "&:hover,&:focus,&.focus": {
        textDecoration: "none",
      },
      "&:focus,&.focus": {
        outline: 0,
        boxShadow: buttonProps.focusBoxShadow,
      },
      // Disabled comes first so active can properly restyle
      "&.disabled,&:disabled": {
        opacity: 0.65,
        boxShadow: "none",
      },

      "&:active,&.active": fuse(
        {backgroundImage: "none"},
        boxShadow(themeProps,`${buttonProps.focusBoxShadow}, ${buttonProps.activeBoxShadow}`),
      ),
    },
  );
}

buttonRule.className = "btn";
export {buttonRule as button};

// Future-proof disabling of clicks on `<a>` elements
// a.btn.disabled,
// fieldset[disabled] a.btn {
//   pointer-events: none;
// }


//
// Alternate buttons
//

export function coloredButton(name, bgColor) {
  const rule = (themeProps)=>{
    return buttonVariant(themeProps,bgColor,bgColor);
  };
  rule.className = `btn-${name}`;
  return rule;
}

export function outlinedButton(name, borderColor, hoverColor) {
  const rule = (themeProps)=>{
    return buttonOutlineVariant(themeProps,borderColor,hoverColor);
  };
  rule.className = `btn-outline-${name}`;
  return rule;
}

export function themedButtons(themeProps = {}) {
  const {colorProps,optionsProps} = getProps(themeProps,{color,options});
  const {themeColors,white,gray900} = colorProps;
  const rules = [];
  each(optionsProps.themeColors,(themeName)=>{
    const themeColor = themeColors[themeName];
    rules.push(coloredButton(themeName,themeColor));
    const outlineHoverColor = themeName === "light" ? gray900 : white;
    rules.push(outlinedButton(themeName,themeColor,outlineHoverColor));
  });
  return rules;
}



//
// Link buttons
//

// Make a button look and behave like a link
// .btn-link {
//   font-weight: $font-weight-normal;
//   color: $link-color;
//   background-color: transparent;
//
//   @include hover {
//     color: $link-hover-color;
//     text-decoration: $link-hover-decoration;
//     background-color: transparent;
//     border-color: transparent;
//   }
//
//   &:focus,
//   &.focus {
//     border-color: transparent;
//     box-shadow: none;
//   }
//
//   &:disabled,
//   &.disabled {
//     color: $btn-link-disabled-color;
//   }
//
//   // No need for an active state here
// }


//
// Button Sizes
//

export function largeButton(themeProps = {}) {

  const {buttonProps,fontProps} = getProps(themeProps,{button,font});

  return buttonSize(themeProps,buttonProps.inputPaddingYLarge, buttonProps.inputPaddingXLarge, fontProps.sizeLarge, buttonProps.inputLineHeightLarge, buttonProps.borderRadiusLarge);
}

largeButton.className = "btn-lg";


export function smallButton(themeProps = {}) {

  const {buttonProps,fontProps} = getProps(themeProps,{button,font});

  return buttonSize(themeProps,buttonProps.inputPaddingYSmall, buttonProps.inputPaddingXSmall, fontProps.sizeSmall, buttonProps.inputLineHeightSmall, buttonProps.borderRadiusSmall);
}

smallButton.className = "btn-sm";

//
// Block button
//

export function blockButton(themeProps) {
  const {buttonProps} = getProps(themeProps,{button});

  const styles = {
    display: "block",
    width: "100%",
    "& + &": {
      marginTop: buttonProps.blockSpacingY,
    }
  };

  // Specificity overrides
  const width100 = {width: "100%"};
  [
    'input[type="submit"]',
    'input[type="reset"]',
    'input[type="button"]',
  ].forEach((input)=>{
    styles[`${input}&`] = width100;
  });

  return styles;
}
blockButton.className = "btn-block";
