import {merge} from "lodash";

import {body,font,link,table,paragraph} from "../variables";
import getProps from "../lib/get-props";

function applyElementStyles(target,elements,styles) {
  const selectors = elements.map((el)=>`& ${el}`);
  merge(target,{
    [selectors.join(", ")]: styles,
  });
}

// Reboot
//
// Normalization of HTML elements, manually forked from Normalize.css to remove
// styles targeting irrelevant browsers while applying new styles.
//
// Normalize is licensed MIT. https://github.com/necolas/normalize.css



export default function reset(themeProps = {}) {

  const {fontProps,bodyProps,linkProps,tableProps,paragraphProps} = getProps(themeProps,{body,font,link,table,paragraph});


  // Document
  //
  // 1. Change from `box-sizing: content-box` so that `width` is not affected by `padding` or `border`.
  // 2. Change the default font family in all browsers.
  // 3. Correct the line height in all browsers.
  // 4. Prevent adjustments of font size after orientation changes in IE on Windows Phone and in iOS.
  // 5. Setting @viewport causes scrollbars to overlap content in IE11 and Edge, so
  //    we force a non-overlapping, non-auto-hiding scrollbar to counteract.
  // 6. Change the default tap highlight to be completely transparent in iOS.

  const styles = {
    boxSizing: "border-box",
    "&::before, &::after": {
      boxSizing: "border-box",
    },
    "& *, & *::before, & *::after": {
      boxSizing: "border-box",
      background: "transparent",
      boxShadow: "none",
    },

    // html
    fontFamily: "sans-serif",
    lineHeight: 1.15,
    WebkitTextSizeAdjust: "100%",
    MsTextSizeAdjust: "100%",
    // MsOverflowStyle: "scrollbar",
    WebkitTapHighlightColor: "rgba(0,0,0,0)",
  };

  // // IE10+ doesn't honor `<meta name="viewport">` in some cases.
  // @at-root {
  //   @-ms-viewport { width: device-width; }
  // }

  // Shim for "new" HTML5 structural elements to display correctly (IE10, older browsers)
  applyElementStyles(styles,["article", "aside", "dialog", "figcaption", "figure", "footer", "header", "hgroup", "main", "nav", "section"],{
    display: "block",
  });

  // remove backgrounds and shadows (not in Bootstrap)
//   applyElementStyles(
//     a,
//     abbr,
//     area,
//     article,
//     aside,
//     audio,
//     b,
//     bdo,
//     blockquote,
//     body,
//     button,
//     canvas,
//     caption,
//     cite,
//     code,
//     col,
//     colgroup,
//     command,
//     datalist,
//     dd,
//     del,
//     details,
//     dialog,
//     dfn,
//     div,
//     dl,
//     dt,
//     em,
//     embed,
//     fieldset,
//     figure,
//     form,
//     h1,
//     h2,
//     h3,
//     h4,
//     h5,
//     h6,
//     head,
//     header,
//     hgroup,
//     hr,
//     html,
//     i,
//     iframe,
//     img,
//     input,
//     ins,
//     keygen,
//     kbd,
//     label,
//     legend,
//     li,
//     map,
//     mark,
//     menu,
//     meter,
//     nav,
//     noscript,
//     object,
//     ol,
//     optgroup,
//     option,
//     output,
//     p,
//     param,
//     pre,
//     progress,
//     q,
//     rp,
//     rt,
//     ruby,
//     samp,
//     section,
//     select,
//     small,
//     span,
//     strong,
//     sub,
//     sup,
//     table,
//     tbody,
//     td,
//     textarea,
//     tfoot,
//     th,
//     thead,
//     time,
//     tr,
//     ul,
//     var,
//     video
//   {
// 	background: transparent;
// 	border: 0;
// 	font-size: 100%;
// 	font: inherit;
// 	margin: 0;
// 	outline: none;
// 	padding: 0;
// 	text-align: left;
// 	text-decoration: none;
// 	vertical-align: baseline;
// 	z-index: 1;
// }


  merge(styles,{

    // Body
    //
    // 1. Remove the margin in all browsers.
    // 2. As a best practice, apply a default `background-color`.
    // 3. Set an explicit initial text-align value so that we can later use the
    //    the `inherit` value on things like `<th>` elements.

    margin: 0,
    fontFamily: fontProps.familyBase,
    fontSize: fontProps.sizeBase,
    fontWeight: fontProps.weightBase,
    lineHeight: fontProps.lineHeightBase,
    color: bodyProps.color,
    textAlign: "left",
    backgroundColor: bodyProps.bg,
  });


  // Reset font-sizes (not in Bootstrap)
  applyElementStyles(styles,['label'],{
    fontSize: fontProps.sizeBase,
  });



  // Suppress the focus outline on elements that cannot be accessed via keyboard.
  // This prevents an unwanted focus outline from appearing around elements that
  // might still respond to pointer events.
  //
  // Credit: https://github.com/suitcss/base
  applyElementStyles(styles,['[tabindex="-1"]:focus'],{
    outline: "none !important",
  });



  // Content grouping
  //
  // 1. Add the correct box sizing in Firefox.
  // 2. Show the overflow in Edge and IE.
  applyElementStyles(styles,["hr"],{
    boxSizing: "content-box",
    height: 0,
    overflow: "visible",
  });

  //
  // Typography
  //

  // Remove top margins from headings
  //
  // By default, `<h1>`-`<h6>` all receive top and bottom margins. We nuke the top
  // margin for easier control within type scales as it avoids margin collapsing.
  applyElementStyles(styles,["h1", "h2", "h3", "h4", "h5", "h6"],{
    marginTop: 0,
    marginBottom: fontProps.headingsMarginBottom,
  });

  // Reset margins on paragraphs
  //
  // Similarly, the top margin on `<p>`s get reset. However, we also reset the
  // bottom margin to use `rem` units instead of `em`.
  applyElementStyles(styles,["p"],{
    marginTop: 0,
    marginBottom: paragraphProps.marginBottom,
  });

  // Abbreviations
  //
  // 1. Remove the bottom border in Firefox 39-.
  // 2. Add the correct text decoration in Chrome, Edge, IE, Opera, and Safari.
  // 3. Add explicit cursor to indicate changed behavior.
  // 4. Duplicate behavior to the data-* attribute for our tooltip plugin

  applyElementStyles(styles,["abbr[title]","abbr[data-original-title]"],{
    // textDecoration: "underline",
    textDecoration: "underline dotted",
    cursor: "help",
    borderBottom: 0,
  });
  applyElementStyles(styles,["address"],{
    marginBottom: "1rem",
    fontStyle: "normal",
    lineHeight: "inherit",
  });

  applyElementStyles(styles,["ol","ul","dl"],{
    marginTop: 0,
    marginBottom: "1rem",
  });

  applyElementStyles(styles,["ol ol","ul ul","ol ul","ul ol"],{
    marginBottom: 0,
  });

  applyElementStyles(styles,["dt"],{
    fontWeight: fontProps.dtWeight,
  });

  applyElementStyles(styles,["dd"],{
    marginBottom: ".5rem",
    marginLeft: 0, // Undo browser default
  });

  applyElementStyles(styles,["blockquote"],{
    margin: "0 0 1rem",
  });

  applyElementStyles(styles,["dfn"],{
    fontStyle: "italic", // Add the correct font style in Android 4.3-
  });

  applyElementStyles(styles,["b","strong"],{
    fontWeight: "bolder", // Add the correct font weight in Chrome, Edge, and Safari
  });

  applyElementStyles(styles,["small"],{
    fontSize: "80%", // Add the correct font size in all browsers
  });

  //
  // Prevent `sub` and `sup` elements from affecting the line height in
  // all browsers.
  //

  applyElementStyles(styles,["sub","sup"],{
    position: "relative",
    fontSize: "75%",
    lineHeight: 0,
    verticalAlign: "baseline",
  });

  applyElementStyles(styles,["sub"],{ bottom: "-.25em" });
  applyElementStyles(styles,["sup"],{ top: "-.5em" });


  //
  // Links
  //

  applyElementStyles(styles,["a"],{
    color: linkProps.color,
    textDecoration: linkProps.decoration,
    backgroundColor: "transparent", // Remove the gray background on active links in IE 10.
    WebkitTextDecorationSkip: "objects", // Remove gaps in links underline in iOS 8+ and Safari 8+.
    "&:hover": {
      color: linkProps.hoverColor,
      textDecoration: linkProps.hoverDecoration,
    },
  });

  // And undo these styles for placeholder links/named anchors (without href)
  // which have not been made explicitly keyboard-focusable (without tabindex).
  // It would be more straightforward to just use a[href] in previous block, but that
  // causes specificity issues in many other styles that are too complex to fix.
  // See https://github.com/twbs/bootstrap/issues/19402

  applyElementStyles(styles,["a:not([href]):not([tabindex])"],{
    color: "inherit",
    textDecoration: "none",

    "&:hover,&:focus": {
      color: "inherit",
      textDecoration: "none",
    },

    "&:focus": {
      outline: 0,
    },
  });


  //
  // Code
  //

  applyElementStyles(styles,["pre","code","kbd","samp"],{
    fontFamily: "monospace", // Correct the inheritance and scaling of font size in all browsers.
    fontSize: "1em", // Correct the odd `em` font sizing in all browsers.
  });

  applyElementStyles(styles,["pre"],{
    // Remove browser default top margin
    marginTop: 0,
    // Reset browser default of `1em` to use `rem`s
    marginBottom: "1rem",
    // Don't allow content to break outside
    overflow: "auto",
    // We have @viewport set which causes scrollbars to overlap content in IE11 and Edge, so
    // we force a non-overlapping, non-auto-hiding scrollbar to counteract.
    MsOverflowStyle: "scrollbar",
  });


  //
  // Figures
  //

  applyElementStyles(styles,["figure"],{
    // Apply a consistent margin strategy (matches our type styles).
    margin: "0 0 1rem",
  });


  //
  // Images and content
  //

  applyElementStyles(styles,["img"],{
    verticalAlign: "middle",
    borderStyle: "none", // Remove the border on images inside links in IE 10-.
  });

  applyElementStyles(styles,["svg:not(:root)"],{
    overflow: "hidden", // Hide the overflow in IE
  });


  // Avoid 300ms click delay on touch devices that support the `touch-action` CSS property.
  //
  // In particular, unlike most other browsers, IE11+Edge on Windows 10 on touch devices and IE Mobile 10-11
  // DON'T remove the click delay when `<meta name="viewport" content="width=device-width">` is present.
  // However, they DO support removing the click delay via `touch-action: manipulation`.
  // See:
  // * https://getbootstrap.com/docs/4.0/content/reboot/#click-delay-optimization-for-touch
  // * https://caniuse.com/#feat=css-touch-action
  // * https://patrickhlauke.github.io/touch/tests/results/#suppressing-300ms-delay

  applyElementStyles(styles,["a",'area','button','[role="button"]','input:not([type=range])','label','select','summary','textarea'],{
    touchAction: "manipulation",
  });


  //
  // Tables
  //

  applyElementStyles(styles,["table"],{
    borderCollapse: "collapse", // Prevent double borders
  });

  applyElementStyles(styles,["caption"],{
    paddingTop: tableProps.cellPadding,
    paddingBottom: tableProps.cellPadding,
    color: fontProps.textMuted,
    textAlign: "left",
    captionSide: "bottom",
  });

  applyElementStyles(styles,["th"],{
    // Matches default `<td>` alignment by inheriting from the `<body>`, or the
    // closest parent with a set `text-align`.
    textAlign: "inherit",
  });


  //
  // Forms
  //

  applyElementStyles(styles,["label"],{
    // Allow labels to use `margin` for spacing.
    display: "inline-block",
    marginBottom: ".5rem",
  });

  // Remove the default `border-radius` that macOS Chrome adds.
  //
  // Details at https://github.com/twbs/bootstrap/issues/24093
  applyElementStyles(styles,["button"],{
    borderRadius: 0,
  });

  // Work around a Firefox/IE bug where the transparent `button` background
  // results in a loss of the default `button` focus styles.
  //
  // Credit: https://github.com/suitcss/base/
  // applyElementStyles(styles,["button:focus"],{
  //   outline: "1px dotted",
  //   outline: "5px auto -webkit-focus-ring-color",
  // });

  applyElementStyles(styles,["input","button","select","optgroup","textarea"],{
    margin: 0, // Remove the margin in Firefox and Safari
    fontFamily: "inherit",
    fontSize: "inherit",
    lineHeight: "inherit",
  });

  applyElementStyles(styles,["button","input"],{
    overflow: "visible", // Show the overflow in Edge
  });

  applyElementStyles(styles,["button","select"],{
    textTransform: "none", // Remove the inheritance of text transform in Firefox
  });

  // 1. Prevent a WebKit bug where (2) destroys native `audio` and `video`
  //    controls in Android 4.
  // 2. Correct the inability to style clickable types in iOS and Safari.
  applyElementStyles(styles,[
    "button",
    'html [type="button"]', // 1
    '[type="reset"]',
    '[type="submit"]',
  ],{
    WebkitAppearance: "button", // 2
  });

  // Remove inner border and padding from Firefox, but don't restore the outline like Normalize.
  applyElementStyles(styles,[
    "button::-moz-focus-inner",
    '[type="button"]::-moz-focus-inner',
    '[type="reset"]::-moz-focus-inner',
    '[type="submit"]::-moz-focus-inner',
  ],{
    padding: 0,
    borderStyle: "none",
  });

  applyElementStyles(styles,['input[type="radio"]','input[type="checkbox"]'],{
    boxSizing: "border-box", // 1. Add the correct box sizing in IE 10-
    padding: 0, // 2. Remove the padding in IE 10-
  });


  applyElementStyles(styles,[
    'input[type="date"]',
    'input[type="time"]',
    'input[type="datetime-local"]',
    'input[type="month"]',
  ],{
    // Remove the default appearance of temporal inputs to avoid a Mobile Safari
    // bug where setting a custom line-height prevents text from being vertically
    // centered within the input.
    // See https://bugs.webkit.org/show_bug.cgi?id=139848
    // and https://github.com/twbs/bootstrap/issues/11266
    WebkitAppearance: "listbox",
  });

  applyElementStyles(styles,["textarea"],{
    overflow: "auto", // Remove the default vertical scrollbar in IE.
    // Textareas should really only resize vertically so they don't break their (horizontal) containers.
    resize: "vertical",
  });

  applyElementStyles(styles,["fieldset"],{
    // Browsers set a default `min-width: min-content;` on fieldsets,
    // unlike e.g. `<div>`s, which have `min-width: 0;` by default.
    // So we reset that to ensure fieldsets behave more like a standard block element.
    // See https://github.com/twbs/bootstrap/issues/12359
    // and https://html.spec.whatwg.org/multipage/#the-fieldset-and-legend-elements
    minWidth: 0,
    // Reset the default outline behavior of fieldsets so they don't affect page layout.
    padding: 0,
    margin: 0,
    border: 0,
  });

  // 1. Correct the text wrapping in Edge and IE.
  // 2. Correct the color inheritance from `fieldset` elements in IE.
  applyElementStyles(styles,["legend"],{
    display: "block",
    width: "100%",
    maxWidth: "100%", // 1
    padding: 0,
    marginBottom: ".5rem",
    fontSize: "1.5rem",
    lineHeight: "inherit",
    color: "inherit", // 2
    whiteSpace: "normal", // 1
  });

  applyElementStyles(styles,["progress"],{
    verticalAlign: "baseline", // Add the correct vertical alignment in Chrome, Firefox, and Opera.
  });

  // Correct the cursor style of increment and decrement buttons in Chrome.
  applyElementStyles(styles,['[type="number"]::-webkit-inner-spin-button','[type="number"]::-webkit-outer-spin-button'],{
    height: "auto",
  });

  applyElementStyles(styles,['[type="search"]'],{
    // This overrides the extra rounded corners on search inputs in iOS so that our
    // `.form-control` class can properly style them. Note that this cannot simply
    // be added to `.form-control` as it's not specific enough. For details, see
    // https://github.com/twbs/bootstrap/issues/11586.
    outlineOffset: "-2px", // 2. Correct the outline style in Safari.
    WebkitAppearance: "none",
  });

  //
  // Remove the inner padding and cancel buttons in Chrome and Safari on macOS.
  //

  applyElementStyles(styles,[
    '[type="search"]::-webkit-search-cancel-button',
    '[type="search"]::-webkit-search-decoration'
  ],{
    WebkitAppearance: "none",
  });

  //
  // 1. Correct the inability to style clickable types in iOS and Safari.
  // 2. Change font properties to `inherit` in Safari.
  //

  applyElementStyles(styles,["::-webkit-file-upload-button"],{
    font: "inherit", // 2
    WebkitAppearance: "button", // 1
  });

  //
  // Correct element displays
  //

  applyElementStyles(styles,["output"],{
    display: "inline-block",
  });

  applyElementStyles(styles,["summary"],{
    display: "list-item", // Add the correct display in all browsers
  });

  applyElementStyles(styles,["template"],{
    display: "none", // Add the correct display in IE
  });

  // Always hide an element with the `hidden` HTML attribute (from PureCSS).
  // Needed for proper display in IE 10-.
  applyElementStyles(styles,["[hidden]"],{
    display: "none !important",
  });

  return styles;
}

reset.className = "root";
